/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var News = function()
{
	var news = {};
	
	/**
		Initialize the page.
	*/
	news.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', news.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', news.validateNews);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'NEWS', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'NEWS'};
			var tableAttributes = {'table-id': 'news'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected);
		}
	}

	// Validate the news information
	news.validateNews = function()
	{
		var dateFormat = Data.get('page-data', 'date-format');
		var frm = document.forms['news'];
		if (Strings.trim(frm.title.value) == '')
		{
			Forms.showNotification(frm.title, 'You must enter a title');
			return false;
		}
		else if (!Dates.isDate(frm.start_date.value, dateFormat))
		{
			Forms.showNotification(frm.start_date, "'"+frm.start_date.value+"' is not a valid date");
			return false;
		}
		else if (!Dates.isDate(frm.news_date.value, dateFormat))
		{
			Forms.showNotification(frm.news_date, "'"+frm.news_date.value+"' is not a valid date");
			return false;
		}
		else if (frm.end_date.value != '' && !Dates.isDate(frm.end_date.value, dateFormat))
		{
			Forms.showNotification(frm.end_date, "'"+frm.end_date.value+"' is not a date");
			return false;
		}
		return true;
	}
	
	// Save the news settings
	news.save = function()
	{
		var frm = document.forms['news'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}

	return news;
}();

Doc.onLoad(News.initPage);
