/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Executor = function()
{
	var executor = {};
	
	var allParameters;
	var currentParameter;
	
	/**
		Initialize the page.
	*/
	executor.initPage = function()
	{
		Buttons.addClickHandler('close', executor.cancelWizard);
		Buttons.addClickHandler('btnSave', executor.saveSettings);
		
		Buttons.addClickHandler('btnShowError', executor.showErrorList);
		Buttons.addClickHandler('btnHideError', executor.hideErrorList);
		
		// Toolbar
		Buttons.addClickHandler('btnAddParameter', executor.addNewParameter);
		Buttons.addClickHandler('btnRemoveParameter', executor.removeParameter);
		Buttons.addClickHandler('btnMoveUp', executor.moveParameter);
		Buttons.addClickHandler('btnMoveDown', executor.moveParameter);
		
		allParameters = Data.json('page-data', 'parameters');
		for (var i = 0; i < allParameters.length; i++)
		{
			executor.addParameterToList(allParameters[i]);
		}
		
		// Parameter list
		Events.addEventHandler('programParameters', 'change', executor.parameterOnClick);

		// Default, min and max values
		Events.addEventHandler('parameterDefault', 'keypress', executor.numericOnly);
		Events.addEventHandler('minValue', 'keypress', executor.numericOnly);
		Events.addEventHandler('maxValue', 'keypress', executor.numericOnly);
		
		// Changes to parameters
		Events.addEventHandler('parameterType', 'change', executor.parameterTypeOnChange);
		Events.addEventHandler('parameterName', 'change', executor.onFormChange);
		Events.addEventHandler('parameterIsRequired', 'click', executor.onFormChange);
		Events.addEventHandler('parameterIsMultiple', 'click', executor.onFormChange);
		Events.addEventHandler('minValue', 'change', executor.onFormChange);
		Events.addEventHandler('maxValue', 'change', executor.onFormChange);
		Events.addEventHandler('enumerationOptions', 'change', executor.onFormChange);
		Events.addEventHandler('parameterDefault', 'change', executor.onFormChange);
		Events.addEventHandler('booleanDefaultTrue', 'click', executor.onFormChange);
		Events.addEventHandler('booleanDefaultFalse', 'click', executor.onFormChange);
		Events.addEventHandler('parameterDescription', 'change', executor.onFormChange);
	}
	
	executor.initElements = function(element, autoInit)
	{
		if (autoInit == 'toggle-stacktrace')
		{
			Buttons.addClickHandler(element, executor.toggleStacktrace);
		}
	}

	/**
		Cancel export wizard.
	*/
	executor.cancelWizard = function()
	{
		location.href = 'index.jsp?ID='+App.getSessionId()+'&cmd=CancelWizard';
	}

	executor.hideErrorList = function()
	{
		Doc.hide('errorlist');
		Doc.show('showerrorlist');
	}
	
	executor.showErrorList = function()
	{
		Doc.show('errorlist');
		Doc.hide('showerrorlist');
	}
	
	executor.toggleStacktrace = function(event)
	{
		var index = Data.get(event.currentTarget, 'stracktrace-index');
		Doc.showHide('stacktrace.' + index);
		var img = event.currentTarget.firstChild;
		if (img.src.indexOf('gonext') > 0)
		{
			img.src = App.getRoot() + '/images/move_down.png';
		}
		else
		{
			img.src = App.getRoot() + '/images/gonext.png';
		}
	}

	executor.addParameterToList = function(parameter)
	{
		var frm = document.forms['configure'];
		var option = new Option(parameter.name + ' ('+parameter.parameterType+')', parameter.name);
		frm.programParameters[frm.programParameters.length] = option;
	}
	
	executor.writeParameterToForm = function(parameter)
	{
		var frm = document.forms['configure'];
		Forms.selectListOption(frm.parameterType, parameter.parameterType);
		frm.parameterName.value = parameter.name;
		frm.parameterIsRequired.checked = parameter.required;
		frm.parameterIsMultiple.checked = parameter.multiple;
		if (parameter.parameterType == 'boolean')
		{
			Forms.checkRadio(frm.booleanDefault, parameter.defaultValue);
		}
		else
		{
			frm.parameterDefault.value = parameter.defaultValue || '';
		}
		frm.enumerationOptions.value = parameter.options['enum'] || '';
		frm.minValue.value = parameter.options['min'] || '';
		frm.maxValue.value = parameter.options['max'] || '';
		frm.parameterDescription.value = parameter.description || '';
	}
	
	executor.readParameterFromForm = function(parameter)
	{
		var frm = document.forms['configure'];
		var type = frm.parameterType.value;
		
		parameter.parameterType = type;
		parameter.name = frm.parameterName.value;
		parameter.required = frm.parameterIsRequired.checked ? 1 : 0;
		parameter.multiple = frm.parameterIsMultiple.checked ? 1 : 0;
		if (type == 'boolean')
		{
			var radio = Forms.getCheckedRadio(frm.booleanDefault);
			parameter.defaultValue = radio ? radio.value : '';
		}
		else
		{
			parameter.defaultValue = frm.parameterDefault.value;
		}
		if (type == 'enumeration')
		{
			parameter.options['enum'] = frm.enumerationOptions.value;
		}
		if (type == 'integer' || type == 'float')
		{
			parameter.options['min'] = frm.minValue.value;
			parameter.options['max'] = frm.maxValue.value;
		}
		parameter.description = frm.parameterDescription.value;
	}

	executor.updateForm = function()
	{
		var frm = document.forms['configure'];
		var type = frm.parameterType.value;
		if (type == 'integer' || type == 'float')
		{
			Doc.show('minMax');
		}
		else
		{
			Doc.hide('minMax');
		}
		if (type == 'boolean')
		{
			Doc.show('booleanDefault');
			Doc.hide('regularDefault');
		}
		else
		{
			Doc.hide('booleanDefault');
			Doc.show('regularDefault');
		}
		if (type == 'enumeration')
		{
			Doc.show('enumerationOptions');
		}
		else
		{
			Doc.hide('enumerationOptions');
		}
	}
	
	executor.parameterTypeOnChange = function()
	{
		executor.updateForm();
		executor.onFormChange();
	}

	
	executor.onFormChange = function()
	{
		var frm = document.forms['configure'];
		var parameter = allParameters[currentParameter];
		executor.readParameterFromForm(parameter);
		var listOption = frm.programParameters[currentParameter];
		listOption.value = parameter.name;
		listOption.text = parameter.name + ' ('+parameter.parameterType+')';
	}
	
	executor.moveParameter = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'move-down');
		var frm = document.forms['configure'];
		var selected = frm.programParameters.selectedIndex;
		Forms.moveListOptions(frm.programParameters, moveDown);
		var moveTo = -1;
		if (moveDown)
		{
			if (selected < allParameters.length - 1) moveTo = selected + 1;
		}
		else 
		{
			if (selected > 0) moveTo = selected - 1;
		}
		if (moveTo >= 0)
		{
			var pp = allParameters[selected];
			allParameters[selected] = allParameters[moveTo];
			allParameters[moveTo] = pp;
		}
	}

	executor.removeParameter = function()
	{
		var frm = document.forms['configure'];
		var selected = frm.programParameters.selectedIndex;
		if (!confirm('Remove ' + allParameters[selected].name + '?')) return;
		frm.programParameters[selected] = null;
		allParameters.splice(selected, 1);
		Doc.hide('parameterDefinition');
	}

	executor.addNewParameter = function()
	{
		var frm = document.forms['configure'];
		var parameter = {'name': 'new', 'parameterType':'string', 'required': 0, 'multiple': 0, 'defaultValue': '', 'description': '', 'options': {}};
		allParameters[allParameters.length] = parameter;
		executor.addParameterToList(parameter);
		frm.programParameters.selectedIndex = frm.programParameters.length-1;
		executor.parameterOnClick();
		frm.parameterName.focus();
		frm.parameterName.select();
	}

	executor.parameterOnClick = function()
	{
		var frm = document.forms['configure'];
		if (frm.programParameters.selectedIndex < 0) return;
		currentParameter = frm.programParameters.selectedIndex;
		var parameter = allParameters[currentParameter];
		if (!parameter) return;
		executor.writeParameterToForm(parameter);
		executor.updateForm();
	}
	
	executor.numericOnly = function(event)
	{
		var frm = document.forms['configure'];
		var type = frm.parameterType.value;
		if (type == 'integer')
		{
			Events.integerOnly(event);
		}
		else if (type == 'float')
		{
			Events.numberOnly(event);
		}
	}
	
	executor.saveSettings = function()
	{
		var frm = document.forms['configure'];
		var xml = '<parameter-definition>\n';
		for (var i=0; i < allParameters.length; i++)
		{
			var parameter = allParameters[i];
			var error = executor.validateParameter(parameter);
			if (error)
			{
				frm.programParameters.selectedIndex = i;
				executor.parameterOnClick();
				Forms.showNotification(error.field, error.msg);
				return;
			}
			xml += executor.parameterToXml(parameter) + '\n';
		}
		xml += '</parameter-definition>\n';
		frm['parameter:externalParameters'].value = xml;
		frm.submit();
	}

	executor.validateParameter = function(parameter)
	{
		var frm = document.forms['configure'];
		
		// A parameter name is required
		if (Strings.trim(parameter.name) == '')
		{
			return new ValidationError('Missing name for parameter', frm.parameterName);
		}
		// A parameter type is required
		else if (Strings.trim(parameter.parameterType) == '')
		{
			return new ValidationError('Missing type for parameter', frm.parameterType);
		}
		// The name must be unique
		for (var i = 0; i < allParameters.length; i++)
		{
			var other = allParameters[i];
			if (other == parameter) break;
			if (other.name == parameter.name)
			{
				return new ValidationError('Parameter has duplicate name: ' + parameter.name, 
						frm.parameterName);
			}
		}
		if (parameter.parameterType == 'enumeration')
		{
			// An enumeration must have at least one option
			var options = Strings.trim(parameter.options['enum']);
			if (options == '')
			{
				return new ValidationError('No options for enumeration: ' + parameter.name, 
						frm.enumerationOptions);
			}
			// If a default value is specified it must be among the options
			else if (parameter.defaultValue)
			{
				var opt = options.split(/[\n\r]/);
				var index = -1;
				for (var i = 0; i < opt.length; i++)
				{
					if (opt[i] == parameter.defaultValue)
					{
						index = i;
						break;
					}
				}
				if (index == -1)
				{
					return new ValidationError("'" + parameter.defaultValue + "' is not an option in enumeration: " + parameter.name, 
							frm.parameterDefault);
				}
			}
		}
		else if (parameter.parameterType == 'integer' || parameter.parameterType == 'float')
		{
			// If a default value is specified, it must be within max and min limits
			if (parameter.defaultValue)
			{
				var def = parseFloat(parameter.defaultValue);
				var min = parseFloat(parameter.options['min']);
				var max = parseFloat(parameter.options['max']);
				if (def < min)
				{
					return new ValidationError('Default value ' + def + ' < ' + min + ' for parameter: ' + parameter.name, 
							frm.parameterDefault);
				}
				else if (def > max)
				{
					return new ValidationError('Default value ' + def + ' > ' + max + ' for parameter: ' + parameter.name, 
							frm.parameterDefault);
				}
			}
		}
		return null;
	}

	executor.parameterToXml = function(parameter)
	{
		var doc = Xml.createDocument('parameter');
		var root = doc.firstChild;
		Xml.setAttribute(root, 'type', parameter.parameterType);
		Xml.setAttribute(root, 'name', parameter.name);
		Xml.setAttribute(root, 'required', parameter.required ? 'true' : 'false');
		Xml.setAttribute(root, 'multiple', parameter.multiple ? 'true' : 'false');
		if (parameter.description) Xml.addTag(root, 'description', parameter.description);
		if (parameter.defaultValue) Xml.addTag(root, 'default', parameter.defaultValue);
		var optionsTag;
		for (var option in parameter.options)
		{
			if (!optionsTag) optionsTag = Xml.addTag(root, 'options');
			var optionTag = Xml.addTag(optionsTag, 'option', parameter.options[option]);
			Xml.setAttribute(optionTag, 'key', option);
		}
		return Xml.toString(doc);
	}
	
	return executor;
}();

Doc.onLoad(Executor.initPage);
Doc.addElementInitializer(Executor.initElements);

function ValidationError(msg, field)
{
	this.msg = msg;
	this.field = field;
}

